@@ -1,13 +1,20 @@
+import inspect
 import json
-import warnings
 from collections.abc import AsyncGenerator, Sequence
 from datetime import datetime, timedelta, timezone
 from os import PathLike
 from pathlib import Path
-from typing import Any, Literal, cast
+from typing import Any, Literal, cast, get_args, get_origin
+
+from funcall import Context
+from pydantic import BaseModel
 
 from lite_agent.agent import Agent
+from lite_agent.chat_display import DisplayConfig
+from lite_agent.chat_display import display_messages as render_chat_messages
+from lite_agent.chat_display import messages_to_string as chat_messages_to_string
 from lite_agent.constants import CompletionMode, StreamIncludes, ToolName
+from lite_agent.context import HistoryContext
 from lite_agent.loggers import logger
 from lite_agent.types import (
     AgentChunk,
@@ -28,8 +35,9 @@ from lite_agent.types import (
     UserInput,
     UserTextContent,
 )
-from lite_agent.types.events import AssistantMessageEvent, FunctionCallOutputEvent, TimingEvent
+from lite_agent.types.events import AssistantMessageEvent, FunctionCallOutputEvent
 from lite_agent.utils.message_builder import MessageBuilder
+from lite_agent.utils.message_state_manager import MessageStateManager
 
 
 class Runner:
@@ -38,50 +46,40 @@ class Runner:
         self.messages: list[FlexibleRunnerMessage] = []
         self.api = api
         self.streaming = streaming
-        self._current_assistant_message: NewAssistantMessage | None = None
+        self._message_state_manager = MessageStateManager()
         self.usage = MessageUsage(input_tokens=0, output_tokens=0, total_tokens=0)
 
-    def _start_assistant_message(self, content: str = "", meta: AssistantMessageMeta | None = None) -> None:
+    async def _start_assistant_message(self, content: str = "", meta: AssistantMessageMeta | None = None) -> None:
         """Start a new assistant message."""
         # Create meta with model information if not provided
         if meta is None:
             meta = AssistantMessageMeta()
             if hasattr(self.agent.client, "model"):
                 meta.model = self.agent.client.model
-        self._current_assistant_message = NewAssistantMessage(
-            content=[AssistantTextContent(text=content)],
-            meta=meta,
-        )
+        await self._message_state_manager.start_message(content, meta)
 
-    def _ensure_current_assistant_message(self) -> NewAssistantMessage:
+    async def _ensure_current_assistant_message(self) -> NewAssistantMessage:
         """Ensure current assistant message exists and return it."""
-        if self._current_assistant_message is None:
-            self._start_assistant_message()
-        if self._current_assistant_message is None:
-            msg = "Failed to create current assistant message"
-            raise RuntimeError(msg)
-        return self._current_assistant_message
+        return await self._message_state_manager.ensure_message_exists()
 
-    def _add_to_current_assistant_message(self, content_item: AssistantTextContent | AssistantToolCall | AssistantToolCallResult) -> None:
+    async def _add_to_current_assistant_message(self, content_item: AssistantTextContent | AssistantToolCall | AssistantToolCallResult) -> None:
         """Add content to the current assistant message."""
-        self._ensure_current_assistant_message().content.append(content_item)
+        if isinstance(content_item, AssistantTextContent):
+            await self._message_state_manager.add_text_delta(content_item.text)
+        elif isinstance(content_item, AssistantToolCall):
+            await self._message_state_manager.add_tool_call(content_item)
+        elif isinstance(content_item, AssistantToolCallResult):
+            await self._message_state_manager.add_tool_result(content_item)
 
-    def _add_text_content_to_current_assistant_message(self, delta: str) -> None:
+    async def _add_text_content_to_current_assistant_message(self, delta: str) -> None:
         """Add text delta to the current assistant message's text content."""
-        message = self._ensure_current_assistant_message()
-        # Find the first text content item and append the delta
-        for content_item in message.content:
-            if content_item.type == "text":
-                content_item.text += delta
-                return
-        # If no text content found, add new text content
-        message.content.append(AssistantTextContent(text=delta))
+        await self._message_state_manager.add_text_delta(delta)
 
-    def _finalize_assistant_message(self) -> None:
+    async def _finalize_assistant_message(self) -> None:
         """Finalize the current assistant message and add it to messages."""
-        if self._current_assistant_message is not None:
-            self.messages.append(self._current_assistant_message)
-            self._current_assistant_message = None
+        finalized_message = await self._message_state_manager.finalize_message()
+        if finalized_message is not None:
+            self.messages.append(finalized_message)
 
     def _add_tool_call_result(self, call_id: str, output: str, execution_time_ms: int | None = None) -> None:
         """Add a tool call result to the last assistant message, or create a new one if needed."""
@@ -111,6 +109,14 @@ class Runner:
         # Note: In the new architecture, we store everything as NewMessage format
         # The conversion to completion format happens when sending to LLM
 
+    def message_history_text(self) -> str:
+        """Return the collected messages as a plain-text transcript."""
+        return chat_messages_to_string(self.messages)
+
+    def display_message_history(self, *, config: DisplayConfig | None = None) -> None:
+        """Render the collected messages using the terminal-friendly display."""
+        render_chat_messages(self.messages, config=config)
+
     def _normalize_includes(self, includes: Sequence[AgentChunkType] | None) -> Sequence[AgentChunkType]:
         """Normalize includes parameter to default if None."""
         return includes if includes is not None else StreamIncludes.DEFAULT_INCLUDES
@@ -119,6 +125,59 @@ class Runner:
         """Normalize record_to parameter to Path object if provided."""
         return Path(record_to) if record_to else None
 
+    def _tool_expects_history_context(self, tool_calls: Sequence["ToolCall"]) -> bool:
+        """Check if any of the tool calls expect HistoryContext in their signatures.
+
+        Returns True if any tool function has a Context[HistoryContext[...]] parameter,
+        False if they expect Context[...] without HistoryContext wrapper.
+        """
+        if not tool_calls:
+            return False
+
+        for tool_call in tool_calls:
+            tool_func = self.agent.fc.function_registry.get(tool_call.function.name)
+            if not tool_func:
+                continue
+
+            # Get function signature
+            sig = inspect.signature(tool_func)
+
+            # Check each parameter for Context annotation
+            for param in sig.parameters.values():
+                if param.annotation == inspect.Parameter.empty:
+                    continue
+
+                # Check if parameter is Context[...]
+                origin = get_origin(param.annotation)
+                if origin is not None:
+                    # Check if it's Context type (compare by string name to handle import differences)
+                    origin_name = getattr(origin, "__name__", str(origin))
+                    if "Context" in origin_name:
+                        args = get_args(param.annotation)
+                        if args:
+                            # Check if the Context contains HistoryContext
+                            inner_type = args[0]
+                            inner_origin = get_origin(inner_type)
+                            if inner_origin is not None:
+                                inner_name = getattr(inner_origin, "__name__", str(inner_origin))
+                                if "HistoryContext" in inner_name:
+                                    logger.debug(f"Tool {tool_call.function.name} expects HistoryContext")
+                                    return True
+                            # Also check for direct HistoryContext class
+                            elif hasattr(inner_type, "__name__") and "HistoryContext" in inner_type.__name__:
+                                logger.debug(f"Tool {tool_call.function.name} expects HistoryContext")
+                                return True
+
+                # Also handle direct annotation checking
+                if hasattr(param.annotation, "__name__"):
+                    annotation_str = str(param.annotation)
+                    if "HistoryContext" in annotation_str:
+                        logger.debug(f"Tool {tool_call.function.name} expects HistoryContext (direct)")
+                        return True
+
+        logger.debug("No tools expect HistoryContext")
+        return False
+
     async def _handle_tool_calls(self, tool_calls: "Sequence[ToolCall] | None", includes: Sequence[AgentChunkType], context: "Any | None" = None) -> AsyncGenerator[AgentChunk, None]:  # noqa: ANN401
         """Handle tool calls and yield appropriate chunks."""
         if not tool_calls:
@@ -127,10 +186,12 @@ class Runner:
         # Check for transfer_to_agent calls first
         transfer_calls = [tc for tc in tool_calls if tc.function.name == ToolName.TRANSFER_TO_AGENT]
         if transfer_calls:
+            logger.info(f"Processing {len(transfer_calls)} transfer_to_agent calls")
             # Handle all transfer calls but only execute the first one
             for i, tool_call in enumerate(transfer_calls):
                 if i == 0:
                     # Execute the first transfer
+                    logger.info(f"Executing agent transfer: {tool_call.function.arguments}")
                     call_id, output = await self._handle_agent_transfer(tool_call)
                     # Generate function_call_output event if in includes
                     if "function_call_output" in includes:
@@ -189,6 +250,34 @@ class Runner:
                         )
             return  # Stop processing other tool calls after transfer
 
+        # Check if tools expect HistoryContext wrapper
+        expects_history = self._tool_expects_history_context(tool_calls)
+
+        if expects_history:
+            # Auto-inject history messages into context for tools that expect HistoryContext
+            if context is not None and not isinstance(context, Context):
+                # If user provided a plain object, wrap it in Context first
+                context = Context(context)
+
+            if isinstance(context, Context):
+                # Extract original value and wrap in HistoryContext
+                original_value = context.value
+                wrapped = HistoryContext(
+                    history_messages=self.messages.copy(),
+                    data=original_value,
+                )
+                context = Context(wrapped)
+            else:
+                # No context provided, create HistoryContext with only history messages
+                wrapped = HistoryContext(history_messages=self.messages.copy())
+                context = Context(wrapped)
+        elif context is not None and not isinstance(context, Context):
+            # Tools don't expect HistoryContext, wrap user object in Context
+            context = Context(context)
+        elif context is None:
+            # Provide empty context for tools that don't expect HistoryContext
+            context = Context(None)
+
         async for tool_call_chunk in self.agent.handle_tool_calls(tool_calls, context=context):
             # if tool_call_chunk.type == "function_call" and tool_call_chunk.type in includes:
             #     yield tool_call_chunk
@@ -205,8 +294,6 @@ class Runner:
                     last_message = cast("NewAssistantMessage", self.messages[-1])
                     last_message.content.append(tool_result)
 
-                # Note: For completion API compatibility, the conversion happens when sending to LLM
-
     async def _collect_all_chunks(self, stream: AsyncGenerator[AgentChunk, None]) -> list[AgentChunk]:
         """Collect all chunks from an async generator into a list."""
         return [chunk async for chunk in stream]
@@ -218,7 +305,7 @@ class Runner:
         includes: Sequence[AgentChunkType] | None = None,
         context: "Any | None" = None,  # noqa: ANN401
         record_to: PathLike | str | None = None,
-        agent_kwargs: dict[str, Any] | None = None,
+        response_format: type[BaseModel] | dict[str, Any] | None = None,
     ) -> AsyncGenerator[AgentChunk, None]:
         """Run the agent and return a RunResponse object that can be asynchronously iterated for each chunk.
 
@@ -231,7 +318,7 @@ class Runner:
         # If no user input provided, use continue logic
         if user_input is None:
             logger.debug("No user input provided, using continue logic")
-            return self._run_continue_stream(max_steps, includes, self._normalize_record_path(record_to), context)
+            return self._run_continue_stream(max_steps, includes, self._normalize_record_path(record_to), context, response_format)
 
         # Cancel any pending tool calls before processing new user input
         # and yield cancellation events if they should be included
@@ -253,7 +340,7 @@ class Runner:
                 # Handle single message (BaseModel, TypedDict, or dict)
                 self.append_message(user_input)  # type: ignore[arg-type]
         logger.debug("Messages prepared, calling _run")
-        return self._run(max_steps, includes, self._normalize_record_path(record_to), context=context, agent_kwargs=agent_kwargs)
+        return self._run(max_steps, includes, self._normalize_record_path(record_to), context=context, response_format=response_format)
 
     async def _run(
         self,
@@ -261,7 +348,7 @@ class Runner:
         includes: Sequence[AgentChunkType],
         record_to: Path | None = None,
         context: Any | None = None,  # noqa: ANN401
-        agent_kwargs: dict[str, Any] | None = None,
+        response_format: type[BaseModel] | dict[str, Any] | None = None,
     ) -> AsyncGenerator[AgentChunk, None]:
         """Run the agent and return a RunResponse object that can be asynchronously iterated for each chunk."""
         logger.debug(f"Running agent with messages: {self.messages}")
@@ -280,34 +367,39 @@ class Runner:
         # Determine completion condition based on agent configuration
         completion_condition = getattr(self.agent, "completion_condition", CompletionMode.STOP)
 
+        # If termination_tools are set but completion_condition is still STOP,
+        # automatically switch to CALL mode to enable custom termination
+        if completion_condition == CompletionMode.STOP and hasattr(self.agent, "termination_tools") and self.agent.termination_tools:
+            completion_condition = CompletionMode.CALL
+            logger.debug(f"Auto-switching to CALL mode due to termination_tools: {self.agent.termination_tools}")
+
         def is_finish() -> bool:
             if completion_condition == CompletionMode.CALL:
-                # Check if wait_for_user was called in the last assistant message
+                # Check if any termination tool was called in the last assistant message
                 if self.messages and isinstance(self.messages[-1], NewAssistantMessage):
                     last_message = self.messages[-1]
                     for content_item in last_message.content:
-                        if isinstance(content_item, AssistantToolCallResult) and self._get_tool_call_name_by_id(content_item.call_id) == ToolName.WAIT_FOR_USER:
-                            return True
+                        if isinstance(content_item, AssistantToolCallResult):
+                            tool_name = self._get_tool_call_name_by_id(content_item.call_id)
+                            # Check custom termination tools first, then default wait_for_user
+                            if hasattr(self.agent, "termination_tools") and self.agent.termination_tools:
+                                if tool_name in self.agent.termination_tools:
+                                    return True
+                            elif tool_name == ToolName.WAIT_FOR_USER:
+                                return True
                 return False
             return finish_reason == CompletionMode.STOP
 
         while not is_finish() and steps < max_steps:
             logger.debug(f"Step {steps}: finish_reason={finish_reason}, is_finish()={is_finish()}")
-            # Convert to legacy format only when needed for LLM communication
-            # This allows us to keep the new format internally but ensures compatibility
-            # Extract agent kwargs for reasoning configuration
-            reasoning = None
-            if agent_kwargs:
-                reasoning = agent_kwargs.get("reasoning")
-
-            logger.debug(f"Using API: {self.api}, streaming: {self.streaming}")
+            logger.info(f"Making LLM request: API={self.api}, streaming={self.streaming}, messages={len(self.messages)}")
             match self.api:
                 case "completion":
                     logger.debug("Calling agent.completion")
                     resp = await self.agent.completion(
                         self.messages,
                         record_to_file=record_to,
-                        reasoning=reasoning,
+                        response_format=response_format,
                         streaming=self.streaming,
                     )
                 case "responses":
@@ -315,7 +407,7 @@ class Runner:
                     resp = await self.agent.responses(
                         self.messages,
                         record_to_file=record_to,
-                        reasoning=reasoning,
+                        response_format=response_format,
                         streaming=self.streaming,
                     )
                 case _:
@@ -331,48 +423,55 @@ class Runner:
                         logger.debug(f"Assistant message chunk: {len(chunk.message.content) if chunk.message.content else 0} content items")
                         # Start or update assistant message in new format
                         # If we already have a current assistant message, just update its metadata
-                        if self._current_assistant_message is not None:
+                        current_message = await self._message_state_manager.get_current_message()
+                        if current_message is not None:
                             # Preserve all existing metadata and only update specific fields
-                            original_meta = self._current_assistant_message.meta
-                            original_meta.sent_at = chunk.message.meta.sent_at
-                            if hasattr(chunk.message.meta, "latency_ms"):
-                                original_meta.latency_ms = chunk.message.meta.latency_ms
-                            if hasattr(chunk.message.meta, "output_time_ms"):
-                                original_meta.total_time_ms = chunk.message.meta.output_time_ms
+                            meta_updates = {"sent_at": chunk.message.meta.sent_at}
+                            # Only include fields of type datetime in meta_updates
+                            # Update int fields separately after update_meta
                             # Preserve other metadata fields like model, usage, etc.
                             for attr in ["model", "usage", "input_tokens", "output_tokens"]:
                                 if hasattr(chunk.message.meta, attr):
-                                    setattr(original_meta, attr, getattr(chunk.message.meta, attr))
+                                    meta_updates[attr] = getattr(chunk.message.meta, attr)
+                            await self._message_state_manager.update_meta(**meta_updates)
+                            # Now update int fields directly if present
+                            if hasattr(chunk.message.meta, "latency_ms"):
+                                await self._message_state_manager.update_meta(latency_ms=chunk.message.meta.latency_ms)
+                            if hasattr(chunk.message.meta, "output_time_ms"):
+                                await self._message_state_manager.update_meta(total_time_ms=chunk.message.meta.output_time_ms)
                         else:
-                            # For non-streaming mode, directly use the complete message from the response handler
-                            self._current_assistant_message = chunk.message
+                            # For non-streaming mode, start with complete message
+                            await self._start_assistant_message(meta=chunk.message.meta)
+                            # Add all content from the chunk message
+                            for content_item in chunk.message.content:
+                                await self._add_to_current_assistant_message(content_item)
 
                         # If model is None, try to get it from agent client
-                        if self._current_assistant_message is not None and self._current_assistant_message.meta.model is None and hasattr(self.agent.client, "model"):
-                            self._current_assistant_message.meta.model = self.agent.client.model
+                        current_message = await self._message_state_manager.get_current_message()
+                        if current_message is not None and current_message.meta.model is None and hasattr(self.agent.client, "model"):
+                            await self._message_state_manager.update_meta(model=self.agent.client.model)
                         # Only yield assistant_message chunk if it's in includes and has content
-                        if chunk.type in includes and self._current_assistant_message is not None:
+                        if chunk.type in includes and current_message is not None:
                             # Create a new chunk with the current assistant message content
                             updated_chunk = AssistantMessageEvent(
-                                message=self._current_assistant_message,
+                                message=current_message,
                             )
                             yield updated_chunk
                     case "content_delta":
                         # Accumulate text content to current assistant message
-                        self._add_text_content_to_current_assistant_message(chunk.delta)
+                        await self._add_text_content_to_current_assistant_message(chunk.delta)
                         # Always yield content_delta chunk if it's in includes
                         if chunk.type in includes:
                             yield chunk
                     case "function_call":
                         logger.debug(f"Function call: {chunk.name}({chunk.arguments or '{}'})")
                         # Add tool call to current assistant message
-                        # Keep arguments as string for compatibility with funcall library
                         tool_call = AssistantToolCall(
                             call_id=chunk.call_id,
                             name=chunk.name,
                             arguments=chunk.arguments or "{}",
                         )
-                        self._add_to_current_assistant_message(tool_call)
+                        await self._add_to_current_assistant_message(tool_call)
                         # Always yield function_call chunk if it's in includes
                         if chunk.type in includes:
                             yield chunk
@@ -390,9 +489,8 @@ class Runner:
                         target_message = None
 
                         # First check if we have a current assistant message
-                        if self._current_assistant_message is not None:
-                            target_message = self._current_assistant_message
-                        else:
+                        target_message = await self._message_state_manager.get_current_message()
+                        if target_message is None:
                             # Otherwise, look for the last assistant message in the list
                             for i in range(len(self.messages) - 1, -1, -1):
                                 current_message = self.messages[i]
@@ -422,9 +520,9 @@ class Runner:
                             yield chunk
                     case "timing":
                         # Update timing information in current assistant message
-                        if self._current_assistant_message is not None:
-                            self._current_assistant_message.meta.latency_ms = chunk.timing.latency_ms
-                            self._current_assistant_message.meta.total_time_ms = chunk.timing.output_time_ms
+                        current_message = await self._message_state_manager.get_current_message()
+                        if current_message is not None:
+                            await self._message_state_manager.update_meta(latency_ms=chunk.timing.latency_ms, total_time_ms=chunk.timing.output_time_ms)
                         # Also try to update the last assistant message if no current message
                         elif self.messages and isinstance(self.messages[-1], NewAssistantMessage):
                             last_message = cast("NewAssistantMessage", self.messages[-1])
@@ -437,7 +535,7 @@ class Runner:
                         yield chunk
 
             # Finalize assistant message so it can be found in pending function calls
-            self._finalize_assistant_message()
+            await self._finalize_assistant_message()
 
             # Check for pending tool calls after processing current assistant message
             pending_tool_calls = self._find_pending_tool_calls()
@@ -463,42 +561,13 @@ class Runner:
         require_confirm_tools = await self.agent.list_require_confirm_tools(tool_calls)
         return bool(require_confirm_tools)
 
-    async def run_continue_until_complete(
-        self,
-        max_steps: int = 20,
-        includes: list[AgentChunkType] | None = None,
-        record_to: PathLike | str | None = None,
-    ) -> list[AgentChunk]:
-        """Deprecated: Use run_until_complete(None) instead."""
-        warnings.warn(
-            "run_continue_until_complete is deprecated. Use run_until_complete(None) instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        resp = self.run_continue_stream(max_steps, includes, record_to=record_to)
-        return await self._collect_all_chunks(resp)
-
-    def run_continue_stream(
-        self,
-        max_steps: int = 20,
-        includes: list[AgentChunkType] | None = None,
-        record_to: PathLike | str | None = None,
-        context: "Any | None" = None,  # noqa: ANN401
-    ) -> AsyncGenerator[AgentChunk, None]:
-        """Deprecated: Use run(None) instead."""
-        warnings.warn(
-            "run_continue_stream is deprecated. Use run(None) instead.",
-            DeprecationWarning,
-            stacklevel=2,
-        )
-        return self._run_continue_stream(max_steps, includes, record_to=record_to, context=context)
-
     async def _run_continue_stream(
         self,
         max_steps: int = 20,
         includes: Sequence[AgentChunkType] | None = None,
         record_to: PathLike | str | None = None,
         context: "Any | None" = None,  # noqa: ANN401
+        response_format: type[BaseModel] | dict[str, Any] | None = None,
     ) -> AsyncGenerator[AgentChunk, None]:
         """Continue running the agent and return a RunResponse object that can be asynchronously iterated for each chunk."""
         includes = self._normalize_includes(includes)
@@ -510,7 +579,7 @@ class Runner:
             tool_calls = self._convert_tool_calls_to_tool_calls(pending_tool_calls)
             async for tool_chunk in self._handle_tool_calls(tool_calls, includes, context=context):
                 yield tool_chunk
-            async for chunk in self._run(max_steps, includes, self._normalize_record_path(record_to)):
+            async for chunk in self._run(max_steps, includes, self._normalize_record_path(record_to), context=context, response_format=response_format):
                 if chunk.type in includes:
                     yield chunk
         else:
@@ -519,7 +588,7 @@ class Runner:
                 msg = "Cannot continue running without a valid last message from the assistant."
                 raise ValueError(msg)
 
-            resp = self._run(max_steps=max_steps, includes=includes, record_to=self._normalize_record_path(record_to), context=context)
+            resp = self._run(max_steps=max_steps, includes=includes, record_to=self._normalize_record_path(record_to), context=context, response_format=response_format)
             async for chunk in resp:
                 yield chunk
 
@@ -529,9 +598,11 @@ class Runner:
         max_steps: int = 20,
         includes: list[AgentChunkType] | None = None,
         record_to: PathLike | str | None = None,
+        context: Any | None = None,  # noqa: ANN401
+        response_format: type[BaseModel] | dict[str, Any] | None = None,
     ) -> list[AgentChunk]:
         """Run the agent until it completes and return the final message."""
-        resp = self.run(user_input, max_steps, includes, record_to=record_to)
+        resp = self.run(user_input, max_steps, includes, record_to=record_to, context=context, response_format=response_format)
         return await self._collect_all_chunks(resp)
 
     def _analyze_last_assistant_message(self) -> tuple[list[AssistantToolCall], dict[str, str]]:
